﻿using System;
using UnityEngine;
using UnityEngine.UI;

[Serializable]
public class StageData
{
    public TextAsset stageData;
    public int mameCount;

    private string[] mapData;

    public char GetData(int x, int y)
    {
        if (mapData == null)
        {
            mapData = stageData.text.Split(new[]
            {
                "\r\n", "\n"
            }, StringSplitOptions.None);
        }
        return mapData[GameMain.GridY - y - 1][x]; //上下が逆になるので、この時点で補正をかける
    }
}

public class GameMain : MonoBehaviour
{
    public GameObject gridPrefab;

    public const int GridX = 9;
    public const int GridY = 9;
    public const int GridSize = 64;
    public Grid[,] field = new Grid[GridY, GridX];

    public UIDialog pauseDialog;
    public UIDialog clearDialog;
    public UIDialog gameOverDialog;
    public Button pauseButton;
    public Button playButton;
    public Button resetButton;
    public StageData[] stageData;

    public Fuku fukuPrefab;
    public Oni oniPrefab;

    public bool IsGamePlay
    {
        set
        {
            _isGamePlay = value;
            pauseButton.enabled = value;
            playButton.enabled = !value;
            resetButton.enabled = !value;
        }
        get { return _isGamePlay; }
    }

    public Text daizuCounText;

    public RideObject.RideType GetRideType(int x, int y)
    {
        if (x < 0 || x >= GridX || y < 0 || y >= GridY)
        {
            return RideObject.RideType.WALL;
        }
        var rideObject = field[y, x].rideObject;
        if (rideObject == null) return RideObject.RideType.NONE;
        return rideObject.Type;
    }

    public Grid GetGrid(int x, int y)
    {
        if (x < 0 || x >= GridX || y < 0 || y >= GridY)
        {
            return null;
        }
        return field[y, x];
    }

    private int _mameCount;
    private bool _isGamePlay;

    public int MameCount
    {
        set
        {
            daizuCounText.text = "x" + value;
            _mameCount = value; 
        }
        get { return _mameCount; }
    }

    private int fukuCount;

    void Start()
    {
        IsGamePlay = false;
        var stage = stageData[stageNo];

        for (int x = 0; x < GridX; ++x)
        {
            for (int y = 0; y < GridY; ++y)
            {
                var obj = Instantiate(gridPrefab);
                obj.transform.localPosition = new Vector3(x*GridSize, y*GridSize, 0);
                obj.transform.localRotation = Quaternion.identity;
                obj.transform.SetParent(this.transform, false);
                field[y, x] = obj.GetComponent<Grid>();

                var code = stage.GetData(x, y);

                if (code == 'f' || code == 'F')
                {
                    var fuku = Instantiate(fukuPrefab);
                    fuku.transform.SetParent(this.transform);
                    fuku.game = this;
                    fuku.Pos = new Vector3(x*GridSize, y*GridSize, 0);
                    fukuCount++;
                }
                if (code == 'o' || code == 'O')
                {
                    var oni = Instantiate(oniPrefab);
                    oni.transform.SetParent(this.transform);
                    oni.game = this;
                    oni.Pos = new Vector3(x*GridSize, y*GridSize, 0);
                }
                else
                {
                    field[y, x].SetCode(code);
                }
            }
        }

        //初期豆数セット
        MameCount = stage.mameCount;

        pauseButton.onClick.AddListener(() =>
        {
            IsGamePlay = false;
            var d = pauseDialog.Show();
            pauseDialog.OnButton.AddListener((arg0, button) =>
            {
                switch (button.ButtonID)
                {
                    case 0:
                        IsGamePlay = true;
                        pauseDialog.Show(false);
                        break;
                    case 1:
                        FadeManager.Instance.LoadLevel("Main", 1.0f);
                        break;
                    case 9:
                        FadeManager.Instance.LoadLevel("Title", 1.0f);
                        break;
                }
            });
        });

        resetButton.onClick.AddListener(() =>
        {
            for (var x = 0; x < 9; ++x)
            {
                for (var y = 0; y < 9; ++y)
                {
                    if (GetRideType(x, y) == RideObject.RideType.DAIZU)
                    {
                        GetGrid(x,y).RemoveRideObject();
                    }
                }
            }
            MameCount = stage.mameCount;
        });

        playButton.onClick.AddListener(() => IsGamePlay = true);
    }

    // Update is called once per frame
    void Update()
    {
        Vector3 mousepos;
        GameUtil.MouseUtil.UnprojectMousePositionXY(out mousepos, Input.mousePosition);

        if (IsGamePlay)
            return;
        var cx = (int) ((mousepos.x )/GridSize);
        var cy = (int) ((mousepos.y + GridSize/2)/GridSize);

        var grid = GetGrid(cx, cy);
        if (grid != null)
        {
            if (Input.GetMouseButton(0) && grid.rideObject == null && MameCount > 0)
            {
                AudioManager.Instance.PlayRegistSE("豆置いた");
                grid.SetCode('m');
                MameCount--;
            }
            if (Input.GetMouseButton(1) && grid.rideObject != null && grid.rideObject.Type==RideObject.RideType.DAIZU)
            {
                grid.RemoveRideObject();
                MameCount++;
            }
        }
    }

    public static int stageNo = 0;
    public void Goal()
    {
        if (--fukuCount <= 0)
        {
            IsGamePlay = false;
            clearDialog.Show();
            clearDialog.OnButton.AddListener((arg0, button) =>
            {
                stageNo = (stageNo + 1) % 5; //TODO 超決め打ちでステージは最大５つまで。　ひどい
                FadeManager.Instance.LoadLevel("Main",1.0f);
            });
        }
    }

    public void GameOver()
    {
        IsGamePlay = false;
        gameOverDialog.Show();
        gameOverDialog.OnButton.AddListener((arg0, button) =>
        {
            FadeManager.Instance.LoadLevel("Main",1.0f);
        });
    }
}