﻿using UnityEngine;
using System.Collections.Generic;
using System.Linq;
using UnityEngine.Audio;

public partial class AudioManager : SingletonMonoBehaviour<AudioManager>
{
    class SoundFormat
    {
        public List<string> soundList = new  List<string>();
        public int playedFrame;
        public int startSample;
        public AudioSource playedAudioSource;

        public void Add(string se)
        {
            soundList.Add(se);
        }
    }

    /// <summary>
    /// 使用するオーディオミキサー。
    /// </summary>
    public AudioMixer Mixer;

    /// <summary>
    /// 使用するオーディオグループ。
    /// </summary>
    public AudioMixerGroup MixerBgmGroup;
    public AudioMixerGroup MixerSeGroup;

    private Dictionary<string, AudioClip> clipList = new Dictionary<string, AudioClip>();
    private AudioSource[] audioSourceList = new AudioSource[20];
    private readonly float pitchUnit = Mathf.Pow(2.0f, 1.0f / 12.0f);

    private const int playableFrameDistance = 5;
    private Dictionary<string, SoundFormat> sndList = new Dictionary<string, SoundFormat>();

    partial void RegisterSe();

    protected void Start()
    {
        RegisterSe();
        IsMute = IsMute;
    }

    public bool IsMute
    {
        set
        {
            Mixer.SetFloat("masterVolume", value ? -80 : 0);
            GameConfig.IsMute = value;
        }
        get { return GameConfig.IsMute; }
    }

    public AudioPlayer AudioBgmPlayers { get; set; }
    public void PlayBGM(AudioClip clip)
    {
        if (AudioBgmPlayers != null && AudioBgmPlayers.Source.clip == clip) return;//すでに同じ曲がBGMになっている場合は再生スキップ
        var mainAuidioPlayer = gameObject.AddComponent<AudioPlayer>();
        mainAuidioPlayer.Source = GetEmptyAudioSource();
        mainAuidioPlayer.Source.outputAudioMixerGroup = MixerBgmGroup;
        mainAuidioPlayer.Source.loop = true;
        mainAuidioPlayer.Source.clip = clip;
        mainAuidioPlayer.IsFade = true;
        mainAuidioPlayer.FadeInSeconds = 1.0f;
        mainAuidioPlayer.Play();
        AudioBgmPlayers = mainAuidioPlayer;
    }

    public void registSE(string name, string se, int startSample = 0)
    {
        if (sndList.ContainsKey(name) == false)
        {
            var snd = new SoundFormat();
            snd.playedFrame = -1000;
            snd.startSample = startSample;
            sndList.Add(name, snd);
        }

        //既にその組み合わせが登録されている場合は抜ける
        if (sndList[name].soundList.Any(s => s == se)) return;

        sndList[name].Add(se);

        //PreLoad
        var temp = Get(se);
    }

    public void PlayRegistSE(string name,bool isOverride = false)
    {
        if (sndList.ContainsKey(name) == false)
        {
            Debug.LogError("Target Sound Not Regist[" + name + "]");
            return; //その名前登録されてないよ
        }
        var snd = sndList[name];
        var soundName = snd.soundList.RandomAt();
        if (isOverride)
        {
            //もし、既に再生しているclipなら、止める
            StopClip(soundName);
        }
        else
        {
            if (Time.frameCount - snd.playedFrame < playableFrameDistance) return; //そんな短いフレームでの再生はゆるざん！
        }
        snd.playedFrame = Time.frameCount;
        NormalPlay(soundName, snd.startSample);
    }

    public void StopRegistSE(string name)
    {
        if (sndList.ContainsKey(name) == false)
        {
            Debug.LogError("Target Sound Not Regist[" + name + "]");
            return; //その名前登録されてないよ
        }
        var snd = sndList[name];
        foreach (var s in snd.soundList)
        {
            StopClip(s);
        }
    }

    private void StopClip(string soundName)
    {
        var clip = Get(soundName);

        var audio = audioSourceList.FirstOrDefault(source => source != null && source.clip == clip);

        if(audio != null)
        {
            audio.Stop();
        }
    }

    private AudioSource GetEmptyAudioSource()
    {
        for (int i = 0; i < audioSourceList.Length; ++i)
        {
            if (audioSourceList[i] == null)
            {
                audioSourceList[i] = gameObject.AddComponent<AudioSource>();
                return audioSourceList[i];
            }
            if (audioSourceList[i].isPlaying == false)
            {
                return audioSourceList[i];
            }
        }
        return null;
    }

    public void Play(AudioSource audio, string soundName, int startSample, int pitch = 0)
    {
        Play(audio,Get(soundName), startSample, pitch);
    }

    public void Play(AudioSource audio, AudioClip clip, int startSample, int pitch = 0)
    {
        if (audio == null) return;
        audio.outputAudioMixerGroup = MixerSeGroup;
        audio.clip = clip;
        audio.pitch = Mathf.Pow(pitchUnit, pitch);
        audio.timeSamples = startSample;
        audio.volume = 1.0f;
        audio.loop = false;
        audio.Play();
    }

    public void NormalPlay(string soundName, int sample, int pitch = 0)
    {
        Play(GetEmptyAudioSource(), soundName, sample, pitch);
    }

    public void Play(string soundName, int pitch = 0)
    {
        Play(GetEmptyAudioSource(), soundName, pitch);
    }

    public void Play(GameObject go, string soundName, int pitch = 0)
    {
        Play(Get(go, soundName), soundName, pitch);
    }

    public AudioSource Get(GameObject go, string soundName)
    {
        var ad = go.GetComponent<AudioSource>();
        if (ad == null || ad.clip.name != soundName)
        {
            ad = go.AddComponent<AudioSource>();
        }
        ad.clip = Get(soundName);
        return ad;
    }

    public AudioClip Get(string soundName)
    {
        if (clipList.ContainsKey(soundName) == false)
        {
            var clip = Resources.Load<AudioClip>(soundName);
            if (clip)
            {
                clipList.Add(soundName, (AudioClip)clip);
            }
            else
            {
                return null;
            }
        }
        return clipList[soundName];
    }
}