using UnityEngine;
using System.Collections;

public class SingletonMonoBehaviour<T> : MonoBehaviour where T : MonoBehaviour{
	private static T sInstance;
	public static T Instance{
		get{
			if(sInstance == null){
				sInstance = FindObjectOfType(typeof(T)) as T;
				if(sInstance == null){
					GameObject obj = new GameObject(typeof(T).Name);
					obj.AddComponent<T>();
                    obj.SendMessage("Initialize", SendMessageOptions.DontRequireReceiver);
                }
            }
			return sInstance;
		}
	}
	
	protected virtual void Awake (){
		if(this == Instance){
			DontDestroyOnLoad(Instance);
			return;
		}
		Destroy(gameObject);	//２つは作らない//
	}
}