using System;
using System.Collections;
using UnityEngine;

namespace Soukoban
{
	public class MoveObjectBase : MonoBehaviour
	{
        public enum MoveDirection
        {
            Up,
            Right,
            Down,
            Left,
            None = -1,
        }

		public enum WalkState
		{
			NONE = -1,
			NORMAL = 0,
			DASH,
			PUSH
		}

		protected const int pushCount = 30; //ړInɓB܂ł̃t[
		protected const int normalCount = 15; //ړInɓB܂ł̃t[
		protected const int dashCount = 10; //ړInɓ܂ł̃t[
		protected int moveCount;
		private Action<MoveObjectBase, MoveDirection> moveEndCallback;
		protected float moveSpeed;
		protected MoveDirection moveState = MoveDirection.None;
		public Vector3 Pos;
		protected WalkState walkState = WalkState.NONE;

		public int Px { get { return (int) ((Pos.x + GameMain.GridSize/2) /GameMain.GridSize); } }
		public int Py { get { return (int) ((Pos.y + GameMain.GridSize/2) /GameMain.GridSize); } }

		public void SetMoveState(MoveDirection state, WalkState wstate, Action<MoveObjectBase, MoveDirection> moveEndCallback = null)
		{
			moveState = state;
			walkState = wstate;
			if (state == MoveDirection.None)
			{
				moveCount = 0;
				moveSpeed = 0;
			}
			else
			{
				//󋵂ɉĈړxς
				if (walkState == WalkState.NORMAL) moveCount = normalCount;
				if (walkState == WalkState.DASH) moveCount = dashCount;
				if (walkState == WalkState.PUSH) moveCount = pushCount;
				moveSpeed = (float)GameMain.GridSize/moveCount;
			}
			this.moveEndCallback = moveEndCallback;
		}

		public void Move()
		{
			if (moveState != MoveDirection.None)
			{
				switch (moveState)
				{
					case MoveDirection.None:
						break;
					case MoveDirection.Right:
						Pos.x += moveSpeed;
						break;
					case MoveDirection.Left:
						Pos.x -= moveSpeed;
						break;
					case MoveDirection.Up:
						Pos.y += moveSpeed;
						break;
					case MoveDirection.Down:
						Pos.y -= moveSpeed;
						break;
				}

				if (--moveCount <= 0) //ړ
				{
					if (moveEndCallback != null) moveEndCallback(this, moveState);
					moveState = MoveDirection.None;
					walkState = WalkState.NONE;
					moveSpeed = 0;
				}
			}
			transform.localPosition = Pos;
		}
	}
}