﻿using System;
using System.Collections;
using Soukoban;
using UnityEngine;

public class Oni : MoveObjectBase
{
    private static readonly int[] animationHash =
    {
        Animator.StringToHash("UpMove@Oni"),
        Animator.StringToHash("RightMove@Oni"),
        Animator.StringToHash("DownMove@Oni"),
        Animator.StringToHash("LeftMove@Oni"),
        Animator.StringToHash("DownMove@Oni"),
    };

    private MoveDirection direction = MoveDirection.Down;

    private Animator _animator;
    private Animator animator
    {
        get { return _animator ?? (_animator = GetComponent<Animator>()); }
    }

    // Use this for initialization
    void Start()
    {
        animator.Play(animationHash.RandomAt());
        StartCoroutine(MoveRoutineIterator());
    }

    //状況判断→移動→移動終了→状況判断・・・を繰り返す
    private IEnumerator MoveRoutineIterator()
    {
        while (true)
        {

            if (game.IsGamePlay == false)
            {
                yield return null;
                continue;
            }
            //状況判断フェイズ。　進行方向と、その先にあるものから次の行動を決定する
            var currentGridType = game.GetRideType(Px, Py);    //現在地のRideType

            if (currentGridType == RideObject.RideType.GOAL)
            {
                AudioManager.Instance.PlayRegistSE("鬼家に入る");
                Destroy(this.gameObject);
                game.GameOver();
            }

            if (currentGridType == RideObject.RideType.DAIZU)
            {
                //DAIZUを消し去り、Uターン？
                var obj = game.GetGrid(Px, Py);
                obj.RemoveRideObject();
                direction = (MoveDirection)(((int)direction + 2) % 4);  //Uターン
                AudioManager.Instance.PlayRegistSE("鬼逃げ");
            }

            for (int checkDirection = 1; checkDirection < 4; checkDirection++) //こうすることでチェックを右・左・後ろに出来る
            {
                RideObject.RideType frontGridType = RideObject.RideType.NONE; //ひとつ前
                switch (direction)
                {
                    case MoveDirection.Up:
                        frontGridType = game.GetRideType(Px, Py + 1);
                        break;
                    case MoveDirection.Right:
                        frontGridType = game.GetRideType(Px + 1, Py);
                        break;
                    case MoveDirection.Down:
                        frontGridType = game.GetRideType(Px, Py - 1);
                        break;
                    case MoveDirection.Left:
                        frontGridType = game.GetRideType(Px - 1, Py);
                        break;
                }

                if (frontGridType == RideObject.RideType.WALL)
                {
                    direction = (MoveDirection)(((int)direction + checkDirection) % 4);
                }
                else
                {
                    break;
                }
            }
            animator.Play(animationHash[(int)direction]);

            SetMoveState(direction, WalkState.NORMAL);

            GetComponent<SpriteRenderer>().sortingOrder = 100 - Py * 2;

            //移動フェイズ　移動が終わるまでまつ
            yield return new WaitWhile(() => moveState != MoveDirection.None);
        }
    }

    public void OnTriggerEnter2D(Collider2D collision)
    {
        game.GameOver();
        Destroy(this.gameObject);
        Destroy(collision.gameObject);
        AudioManager.Instance.PlayRegistSE("鬼と福が当たる");
    }

    //超重要ゲームインスタンス
    public GameMain game;

    void Update()
    {
        Move();
    }
}